@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 #
-# Copyright (c) 2012-2020 Snowflake Computing Inc. All right reserved.
+# Copyright (c) 2012-2021 Snowflake Computing Inc. All right reserved.
 #
 
 import logging
@@ -13,7 +13,12 @@ from typing import Optional
 from uuid import uuid4
 
 from .compat import urlencode
-from .constants import HTTP_HEADER_ACCEPT, HTTP_HEADER_CONTENT_TYPE, HTTP_HEADER_SERVICE_NAME, HTTP_HEADER_USER_AGENT
+from .constants import (
+    HTTP_HEADER_ACCEPT,
+    HTTP_HEADER_CONTENT_TYPE,
+    HTTP_HEADER_SERVICE_NAME,
+    HTTP_HEADER_USER_AGENT,
+)
 from .errors import ForbiddenError, ProgrammingError, ServiceUnavailableError
 from .network import (
     ACCEPT_TYPE_APPLICATION_SNOWFLAKE,
@@ -23,7 +28,7 @@ from .network import (
 )
 
 logger = logging.getLogger(__name__)
-URL = '/incidents/v2/create-incident'
+URL = "/incidents/v2/create-incident"
 CLS_BLACKLIST = frozenset({ProgrammingError})
 
 current_os_release = platform.system()
@@ -31,38 +36,47 @@ current_os_version = platform.release()
 
 
 class Incident(object):
-
-    def __init__(self,
-                 job_id: Optional[str],
-                 request_id: Optional[str],
-                 driver: Optional[str],
-                 driver_version: Optional[str],
-                 error_message: Optional[str],
-                 error_stack_trace: Optional[str],
-                 os: str = current_os_release,
-                 os_version: str = current_os_version):
+    def __init__(
+        self,
+        job_id: Optional[str],
+        request_id: Optional[str],
+        driver: Optional[str],
+        driver_version: Optional[str],
+        error_message: Optional[str],
+        error_stack_trace: Optional[str],
+        os: str = current_os_release,
+        os_version: str = current_os_version,
+    ):
         self.uuid = str(uuid4())
-        self.createdOn = str(datetime.utcnow())[:-3]  # utcnow returns 6 ms digits, we only want 3
+        self.createdOn = str(datetime.utcnow())[
+            :-3
+        ]  # utcnow returns 6 ms digits, we only want 3
         self.jobId = str(job_id)
         self.requestId = str(request_id)
         self.errorMessage = str(error_message)
         self.errorStackTrace = str(error_stack_trace)
         self.os = str(os)
         self.osVersion = str(os_version)
-        self.signature = str(self.__generate_signature(error_message, error_stack_trace))
+        self.signature = str(
+            self.__generate_signature(error_message, error_stack_trace)
+        )
         self.driver = str(driver)
         self.driverVersion = str(driver_version)
 
     def to_dict(self):
-        ret = {"Tags": [{"Name": "driver", "Value": self.driver},
-                         {"Name": "version", "Value": self.driverVersion}],
-               "Name": self.signature,
-               "UUID": self.uuid,
-               "Created_On": self.createdOn,
-               "Value": {
-                   "exceptionMessage": self.errorMessage,
-                   "exceptionStackTrace": self.errorStackTrace
-               }}
+        ret = {
+            "Tags": [
+                {"Name": "driver", "Value": self.driver},
+                {"Name": "version", "Value": self.driverVersion},
+            ],
+            "Name": self.signature,
+            "UUID": self.uuid,
+            "Created_On": self.createdOn,
+            "Value": {
+                "exceptionMessage": self.errorMessage,
+                "exceptionStackTrace": self.errorStackTrace,
+            },
+        }
         # Add optional values
         if self.os:
             ret["Tags"].append({"Name": "os", "Value": self.os})
@@ -81,7 +95,9 @@ class Incident(object):
         return "Incident {id}".format(id=self.uuid)
 
     @staticmethod
-    def __generate_signature(error_message: Optional[str], error_stack_trace: Optional[str]) -> Optional[str]:
+    def __generate_signature(
+        error_message: Optional[str], error_stack_trace: Optional[str]
+    ) -> Optional[str]:
         """Automatically generates signature of Incident."""
         return error_message
 
@@ -92,7 +108,9 @@ class IncidentAPI(object):
     def __init__(self, rest):
         self._rest = rest
 
-    def report_incident(self, incident=None, job_id=None, request_id=None, session_parameters=None):
+    def report_incident(
+        self, incident=None, job_id=None, request_id=None, session_parameters=None
+    ):
         """Reports an incident created.
 
             Example usage:
@@ -126,40 +144,56 @@ class IncidentAPI(object):
         if incident is None:
             cls, exc, _ = exc_info()
             if cls in CLS_BLACKLIST:
-                logger.warning("Ignoring blacklisted exception type: {type}".format(type=cls))
+                logger.warning(
+                    "Ignoring blacklisted exception type: {type}".format(type=cls)
+                )
                 return
-            incident = Incident(job_id,
-                                request_id,
-                                self._rest._connection._internal_application_name,
-                                self._rest._connection._internal_application_version,
-                                str(exc),
-                                format_exc())
+            incident = Incident(
+                job_id,
+                request_id,
+                self._rest._connection._internal_application_name,
+                self._rest._connection._internal_application_version,
+                str(exc),
+                format_exc(),
+            )
 
         if session_parameters is None:
             session_parameters = {}
-        headers = {HTTP_HEADER_CONTENT_TYPE: CONTENT_TYPE_APPLICATION_JSON,
-                   HTTP_HEADER_ACCEPT: ACCEPT_TYPE_APPLICATION_SNOWFLAKE,
-                   HTTP_HEADER_USER_AGENT: PYTHON_CONNECTOR_USER_AGENT}
+        headers = {
+            HTTP_HEADER_CONTENT_TYPE: CONTENT_TYPE_APPLICATION_JSON,
+            HTTP_HEADER_ACCEPT: ACCEPT_TYPE_APPLICATION_SNOWFLAKE,
+            HTTP_HEADER_USER_AGENT: PYTHON_CONNECTOR_USER_AGENT,
+        }
         if HTTP_HEADER_SERVICE_NAME in session_parameters:
-            headers[HTTP_HEADER_SERVICE_NAME] = session_parameters[HTTP_HEADER_SERVICE_NAME]
+            headers[HTTP_HEADER_SERVICE_NAME] = session_parameters[
+                HTTP_HEADER_SERVICE_NAME
+            ]
         body = incident.to_dict()
         logger.debug("Going to report incident with body: {}".format(body))
         try:
             ret = self._rest.request(
-                '/incidents/v2/create-incident?' + urlencode({REQUEST_ID: uuid4()}),
-                body, _include_retry_params=True)
+                "/incidents/v2/create-incident?" + urlencode({REQUEST_ID: uuid4()}),
+                body,
+                _include_retry_params=True,
+            )
         except (ForbiddenError, ServiceUnavailableError):
-            logger.error("Unable to reach endpoint to report incident at url: '{url}' with headers='{headers}' "
-                         "and body: '{body}'".format(url=URL,
-                                                     headers=headers,
-                                                     body=body))
+            logger.error(
+                "Unable to reach endpoint to report incident at url: '{url}' with headers='{headers}' "
+                "and body: '{body}'".format(url=URL, headers=headers, body=body)
+            )
             raise
-        if not ret['success']:
-            logger.warning("Reporting incident failed for reason: '{reason}'".format(reason=ret))
+        if not ret["success"]:
+            logger.warning(
+                "Reporting incident failed for reason: '{reason}'".format(reason=ret)
+            )
             return
-        new_incident_id = ret['data']['incidentId'] if ret.get('data') else None
+        new_incident_id = ret["data"]["incidentId"] if ret.get("data") else None
         if not new_incident_id:
             logger.debug("Reported incident was ignored")
         else:
-            logger.info("Incident has been reported with new incident id: {}".format(ret['data']['incidentId']))
+            logger.info(
+                "Incident has been reported with new incident id: {}".format(
+                    ret["data"]["incidentId"]
+                )
+            )
         return new_incident_id
